<?php
/**
 * @package   ats
 * @copyright Copyright (c)2011-2019 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU GPL v3 or later
 */

namespace Akeeba\TicketSystem\Site\View\Latests;

use Akeeba\TicketSystem\Admin\Helper\Permissions;
use Akeeba\TicketSystem\Admin\Helper\Select;
use Akeeba\TicketSystem\Site\Model\Tickets;
use JText;

defined('_JEXEC') or die;

class Html extends \FOF30\View\DataView\Html
{
	/** @var bool Is the current user a manager? */
	protected $isManager = false;

	/** @var Tickets[] */
	public $items;

	/**
	 * Joomla! menu item ID
	 *
	 * @var   int
	 * @since 3.0.0
	 */
	public $itemID;

	public $filterStatus;
	public $filterCategory;

	protected function onBeforeBrowse()
	{
		/** @var Tickets $model */
		$model                = $this->getModel();
		$this->filterStatus   = $model->getState('status', 'O');
		$this->filterCategory = $model->getState('category', null);
		$allowedCategories    = Permissions::getManagerCategories();

		if (!empty($this->filterCategory) && !in_array($this->filterCategory, $allowedCategories))
		{
			$this->container->platform->raiseError(403, JText::_('JLIB_APPLICATION_ERROR_ACCESS_FORBIDDEN'));
		}

		// Apply custom filters for frontend
		$model->categories($allowedCategories)
			->enabled(1)
			->status($this->filterStatus)
			->filterNewest(1)
			->filter_order_Dir('ASC');

		parent::onBeforeBrowse();

		$this->itemID = $this->input->getInt('Itemid', 0);

		// Add filters to pagination output
		$this->pagination->setAdditionalUrlParam('status', $this->getModel()->getState('status', 'O'));

		if (!empty($this->filterCategory))
		{
			$this->pagination->setAdditionalUrlParam('category', $this->filterCategory);
		}
	}
}
